% SOLVE_MILP Pass MILP to solver
%    [x, fval, exitflag] = solve_milp(f, A, b, Aeq, beq, lb, ub, vartype, sense, options)
%    Select SOLVER by uncommenting.
function [x, fval, exitflag] = solve_milp(f, A, b, Aeq, beq, lb, ub, vartype, sense, options)

% SOLVER = 'GLPK';
% SOLVER = 'SCIP';
% SOLVER = 'CPLEX';
SOLVER = 'Gurobi';

if nargin < 10
    options = optimset('Display', 'final');
end
if nargin < 9
    sense = 1;
end
if nargin < 8
    vartype = char('C' * ones(size(f)));
end
if nargin < 6
    lb = [];
    ub = [];
end
if nargin < 4
    Aeq = [];
    beq = [];
end

switch SOLVER
    case 'GLPK'
        param.tmlim = -1;
        param.save = 0;
        switch options.Display
            case 'off'
                param.msglev = 0;
            case 'iter'
                param.msglev = 3;
            case 'final'
                param.msglev = 2;
            case 'notify'
                param.msglev = 1;
        end
        
        ctype = [ char('U' * ones(size(A, 1), 1)); char ('S' * ones(size(Aeq, 1), 1)) ];
        [x, fval, status] = glpk(f, [A; Aeq], [b; beq], lb, ub, ctype, vartype, sense, param);
        
        if status == 180 || status == 151 || status == 171
            exitflag = 1;
        else
            exitflag = -1;
        end
    case 'SCIP'
        param = struct();        
        switch options.Display
            case 'off'
                param.msglev = 0;
            case 'iter'
                param.msglev = 3;
            case 'final'
                param.msglev = 2;
            case 'notify'
                param.msglev = 1;
        end
        
        ctype = [ char('U' * ones(size(A, 1), 1)); char ('S' * ones(size(Aeq, 1), 1)) ];
        if all(vartype == 'C')
            param.tmlim = -1;
            param.save = 0;
            [x, fval, exitflag] = glpk(f, [A; Aeq], [b; beq], lb, ub, ctype, vartype, sense, param);        
        else
            [x, fval, exitflag] = scip(f, [A; Aeq], [b; beq], lb, ub, ctype, vartype, sense, param);        
        end
    case 'CPLEX'
        cpoptions = cplexoptimset('Display', options.Display);
        if ~strcmp(options.Display, 'off')
            cpoptions = cplexoptimset('Diagnostics', 'on');
        end
        
        if all(vartype == 'C')
            [x, fval, exitflag] = cplexlp(sense * f, A, b, Aeq, beq, lb, ub, [], cpoptions);
            fval = sense * fval;
        else
            vartype = reshape(vartype, 1, []);
            [x, fval, exitflag] = cplexmilp(sense * f, A, b, Aeq, beq, [], [], [], lb, ub, vartype, [], cpoptions);
            fval = sense * fval;
        end
    case 'Gurobi'
        goptions = struct();
        goptions.FeasibilityTol = 1e-9;
        goptions.IntFeasTol = 1e-9;
        switch options.Display
            case 'off'
                goptions.Display = 0;
                goptions.DisplayInterval = 0;
            case 'iter'
                goptions.Display = 2;
                goptions.DisplayInterval = 5;
            case 'final'
                goptions.Display = 2;
                goptions.DisplayInterval = 0;
            case 'notify'
                goptions.Display = 1;
                goptions.DisplayInterval = 0;
        end
        
        contypes = [ char('<' * ones(size(A, 1), 1)); char ('=' * ones(size(Aeq, 1), 1)) ];
        [x, fval, flag] = gurobi_mex(f, sense, [A; Aeq], [b; beq], contypes, lb, ub, vartype, goptions);
        
        if flag == 2
            exitflag = 1;
        else
            exitflag = -1;
        end
end

end
